# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..requests.create_agent_request_configuration_endpoint import CreateAgentRequestConfigurationEndpointParams
from ..requests.create_agent_request_template_variables_value import CreateAgentRequestTemplateVariablesValueParams
from ..requests.create_agent_request_tools_item import CreateAgentRequestToolsItemParams
from ..requests.task import TaskParams
from ..types.basic_error import BasicError
from ..types.create_agent_request_audio_format import CreateAgentRequestAudioFormat
from ..types.create_agent_request_background_noise import CreateAgentRequestBackgroundNoise
from ..types.language_code import LanguageCode
from .requests.update_agent_request_configuration_endpoint import UpdateAgentRequestConfigurationEndpointParams
from .requests.update_agent_request_template_variables_value import UpdateAgentRequestTemplateVariablesValueParams
from .requests.update_agent_request_tools_item import UpdateAgentRequestToolsItemParams
from .types.agents_create_response import AgentsCreateResponse
from .types.agents_delete_response import AgentsDeleteResponse
from .types.agents_get_response import AgentsGetResponse
from .types.agents_list_response import AgentsListResponse
from .types.agents_update_response import AgentsUpdateResponse
from .types.agents_upsert_response import AgentsUpsertResponse
from .types.update_agent_request_audio_format import UpdateAgentRequestAudioFormat
from .types.update_agent_request_background_noise import UpdateAgentRequestBackgroundNoise

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAgentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AgentsListResponse]:
        """
        Returns all agents in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list agents for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentsListResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "agents",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsListResponse,
                    parse_obj_as(
                        type_=AgentsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        background_noise_level: typing.Optional[float] = OMIT,
        background_noise: typing.Optional[CreateAgentRequestBackgroundNoise] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]] = OMIT,
        tasks: typing.Optional[typing.Sequence[TaskParams]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        languages: typing.Optional[typing.Sequence[LanguageCode]] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpointParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentsCreateResponse]:
        """
        Creates a new agent in a project.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project to create the agent in.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        background_noise_level : typing.Optional[float]
            The background noise level of the agent.

        background_noise : typing.Optional[CreateAgentRequestBackgroundNoise]
            The background noise type. Can be "office", "call-center", "coffee-shop", or null.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[TaskParams]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        languages : typing.Optional[typing.Sequence[LanguageCode]]
            Array of ISO 639-1 language codes that the agent should be able to recognize

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpointParams]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentsCreateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "agents",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "project": project,
            },
            json={
                "name": name,
                "phone_number": phone_number,
                "timezone": timezone,
                "voice_id": voice_id,
                "audio_format": audio_format,
                "audio_speed": audio_speed,
                "background_noise_level": background_noise_level,
                "background_noise": background_noise,
                "welcome_message": welcome_message,
                "system_prompt": system_prompt,
                "template_variables": convert_and_respect_annotation_metadata(
                    object_=template_variables,
                    annotation=typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams],
                    direction="write",
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools, annotation=typing.Sequence[CreateAgentRequestToolsItemParams], direction="write"
                ),
                "tasks": convert_and_respect_annotation_metadata(
                    object_=tasks, annotation=typing.Sequence[TaskParams], direction="write"
                ),
                "no_input_poke_sec": no_input_poke_sec,
                "no_input_poke_text": no_input_poke_text,
                "no_input_end_conversation_sec": no_input_end_conversation_sec,
                "languages": languages,
                "boosted_keywords": boosted_keywords,
                "configuration_endpoint": convert_and_respect_annotation_metadata(
                    object_=configuration_endpoint,
                    annotation=typing.Optional[CreateAgentRequestConfigurationEndpointParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsCreateResponse,
                    parse_obj_as(
                        type_=AgentsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def upsert(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        background_noise_level: typing.Optional[float] = OMIT,
        background_noise: typing.Optional[CreateAgentRequestBackgroundNoise] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]] = OMIT,
        tasks: typing.Optional[typing.Sequence[TaskParams]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        languages: typing.Optional[typing.Sequence[LanguageCode]] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpointParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentsUpsertResponse]:
        """
        Upserts an agent by name. If an agent with the same name already exists, it will be updated. Otherwise, it will be created.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project containing the agent.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        background_noise_level : typing.Optional[float]
            The background noise level of the agent.

        background_noise : typing.Optional[CreateAgentRequestBackgroundNoise]
            The background noise type. Can be "office", "call-center", "coffee-shop", or null.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[TaskParams]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        languages : typing.Optional[typing.Sequence[LanguageCode]]
            Array of ISO 639-1 language codes that the agent should be able to recognize

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpointParams]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentsUpsertResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "agents/upsert",
            base_url=self._client_wrapper.get_environment().base,
            method="PUT",
            params={
                "project": project,
            },
            json={
                "name": name,
                "phone_number": phone_number,
                "timezone": timezone,
                "voice_id": voice_id,
                "audio_format": audio_format,
                "audio_speed": audio_speed,
                "background_noise_level": background_noise_level,
                "background_noise": background_noise,
                "welcome_message": welcome_message,
                "system_prompt": system_prompt,
                "template_variables": convert_and_respect_annotation_metadata(
                    object_=template_variables,
                    annotation=typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams],
                    direction="write",
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools, annotation=typing.Sequence[CreateAgentRequestToolsItemParams], direction="write"
                ),
                "tasks": convert_and_respect_annotation_metadata(
                    object_=tasks, annotation=typing.Sequence[TaskParams], direction="write"
                ),
                "no_input_poke_sec": no_input_poke_sec,
                "no_input_poke_text": no_input_poke_text,
                "no_input_end_conversation_sec": no_input_end_conversation_sec,
                "languages": languages,
                "boosted_keywords": boosted_keywords,
                "configuration_endpoint": convert_and_respect_annotation_metadata(
                    object_=configuration_endpoint,
                    annotation=typing.Optional[CreateAgentRequestConfigurationEndpointParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsUpsertResponse,
                    parse_obj_as(
                        type_=AgentsUpsertResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentsGetResponse]:
        """
        Returns an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to get.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentsGetResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsGetResponse,
                    parse_obj_as(
                        type_=AgentsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentsDeleteResponse]:
        """
        Deletes an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to delete.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentsDeleteResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsDeleteResponse,
                    parse_obj_as(
                        type_=AgentsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[UpdateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        background_noise_level: typing.Optional[float] = OMIT,
        background_noise: typing.Optional[UpdateAgentRequestBackgroundNoise] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValueParams]] = OMIT,
        tools: typing.Optional[typing.Sequence[UpdateAgentRequestToolsItemParams]] = OMIT,
        tasks: typing.Optional[typing.Sequence[TaskParams]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        languages: typing.Optional[typing.Sequence[LanguageCode]] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[UpdateAgentRequestConfigurationEndpointParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentsUpdateResponse]:
        """
        Updates an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to update.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[UpdateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        background_noise_level : typing.Optional[float]
            The background noise level of the agent.

        background_noise : typing.Optional[UpdateAgentRequestBackgroundNoise]
            The background noise type. Can be "office", "call-center", "coffee-shop", or null.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValueParams]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[UpdateAgentRequestToolsItemParams]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[TaskParams]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        languages : typing.Optional[typing.Sequence[LanguageCode]]
            Array of ISO 639-1 language codes that the agent should be able to recognize

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[UpdateAgentRequestConfigurationEndpointParams]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentsUpdateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            params={
                "project": project,
            },
            json={
                "name": name,
                "phone_number": phone_number,
                "timezone": timezone,
                "voice_id": voice_id,
                "audio_format": audio_format,
                "audio_speed": audio_speed,
                "background_noise_level": background_noise_level,
                "background_noise": background_noise,
                "welcome_message": welcome_message,
                "system_prompt": system_prompt,
                "template_variables": convert_and_respect_annotation_metadata(
                    object_=template_variables,
                    annotation=typing.Dict[str, UpdateAgentRequestTemplateVariablesValueParams],
                    direction="write",
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools, annotation=typing.Sequence[UpdateAgentRequestToolsItemParams], direction="write"
                ),
                "tasks": convert_and_respect_annotation_metadata(
                    object_=tasks, annotation=typing.Sequence[TaskParams], direction="write"
                ),
                "no_input_poke_sec": no_input_poke_sec,
                "no_input_poke_text": no_input_poke_text,
                "no_input_end_conversation_sec": no_input_end_conversation_sec,
                "languages": languages,
                "boosted_keywords": boosted_keywords,
                "configuration_endpoint": convert_and_respect_annotation_metadata(
                    object_=configuration_endpoint,
                    annotation=typing.Optional[UpdateAgentRequestConfigurationEndpointParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsUpdateResponse,
                    parse_obj_as(
                        type_=AgentsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAgentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AgentsListResponse]:
        """
        Returns all agents in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list agents for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentsListResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agents",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsListResponse,
                    parse_obj_as(
                        type_=AgentsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        background_noise_level: typing.Optional[float] = OMIT,
        background_noise: typing.Optional[CreateAgentRequestBackgroundNoise] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]] = OMIT,
        tasks: typing.Optional[typing.Sequence[TaskParams]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        languages: typing.Optional[typing.Sequence[LanguageCode]] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpointParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentsCreateResponse]:
        """
        Creates a new agent in a project.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project to create the agent in.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        background_noise_level : typing.Optional[float]
            The background noise level of the agent.

        background_noise : typing.Optional[CreateAgentRequestBackgroundNoise]
            The background noise type. Can be "office", "call-center", "coffee-shop", or null.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[TaskParams]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        languages : typing.Optional[typing.Sequence[LanguageCode]]
            Array of ISO 639-1 language codes that the agent should be able to recognize

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpointParams]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentsCreateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agents",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "project": project,
            },
            json={
                "name": name,
                "phone_number": phone_number,
                "timezone": timezone,
                "voice_id": voice_id,
                "audio_format": audio_format,
                "audio_speed": audio_speed,
                "background_noise_level": background_noise_level,
                "background_noise": background_noise,
                "welcome_message": welcome_message,
                "system_prompt": system_prompt,
                "template_variables": convert_and_respect_annotation_metadata(
                    object_=template_variables,
                    annotation=typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams],
                    direction="write",
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools, annotation=typing.Sequence[CreateAgentRequestToolsItemParams], direction="write"
                ),
                "tasks": convert_and_respect_annotation_metadata(
                    object_=tasks, annotation=typing.Sequence[TaskParams], direction="write"
                ),
                "no_input_poke_sec": no_input_poke_sec,
                "no_input_poke_text": no_input_poke_text,
                "no_input_end_conversation_sec": no_input_end_conversation_sec,
                "languages": languages,
                "boosted_keywords": boosted_keywords,
                "configuration_endpoint": convert_and_respect_annotation_metadata(
                    object_=configuration_endpoint,
                    annotation=typing.Optional[CreateAgentRequestConfigurationEndpointParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsCreateResponse,
                    parse_obj_as(
                        type_=AgentsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def upsert(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        background_noise_level: typing.Optional[float] = OMIT,
        background_noise: typing.Optional[CreateAgentRequestBackgroundNoise] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]] = OMIT,
        tasks: typing.Optional[typing.Sequence[TaskParams]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        languages: typing.Optional[typing.Sequence[LanguageCode]] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpointParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentsUpsertResponse]:
        """
        Upserts an agent by name. If an agent with the same name already exists, it will be updated. Otherwise, it will be created.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project containing the agent.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        background_noise_level : typing.Optional[float]
            The background noise level of the agent.

        background_noise : typing.Optional[CreateAgentRequestBackgroundNoise]
            The background noise type. Can be "office", "call-center", "coffee-shop", or null.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItemParams]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[TaskParams]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        languages : typing.Optional[typing.Sequence[LanguageCode]]
            Array of ISO 639-1 language codes that the agent should be able to recognize

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpointParams]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentsUpsertResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agents/upsert",
            base_url=self._client_wrapper.get_environment().base,
            method="PUT",
            params={
                "project": project,
            },
            json={
                "name": name,
                "phone_number": phone_number,
                "timezone": timezone,
                "voice_id": voice_id,
                "audio_format": audio_format,
                "audio_speed": audio_speed,
                "background_noise_level": background_noise_level,
                "background_noise": background_noise,
                "welcome_message": welcome_message,
                "system_prompt": system_prompt,
                "template_variables": convert_and_respect_annotation_metadata(
                    object_=template_variables,
                    annotation=typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams],
                    direction="write",
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools, annotation=typing.Sequence[CreateAgentRequestToolsItemParams], direction="write"
                ),
                "tasks": convert_and_respect_annotation_metadata(
                    object_=tasks, annotation=typing.Sequence[TaskParams], direction="write"
                ),
                "no_input_poke_sec": no_input_poke_sec,
                "no_input_poke_text": no_input_poke_text,
                "no_input_end_conversation_sec": no_input_end_conversation_sec,
                "languages": languages,
                "boosted_keywords": boosted_keywords,
                "configuration_endpoint": convert_and_respect_annotation_metadata(
                    object_=configuration_endpoint,
                    annotation=typing.Optional[CreateAgentRequestConfigurationEndpointParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsUpsertResponse,
                    parse_obj_as(
                        type_=AgentsUpsertResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentsGetResponse]:
        """
        Returns an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to get.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentsGetResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsGetResponse,
                    parse_obj_as(
                        type_=AgentsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentsDeleteResponse]:
        """
        Deletes an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to delete.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentsDeleteResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsDeleteResponse,
                    parse_obj_as(
                        type_=AgentsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[UpdateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        background_noise_level: typing.Optional[float] = OMIT,
        background_noise: typing.Optional[UpdateAgentRequestBackgroundNoise] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValueParams]] = OMIT,
        tools: typing.Optional[typing.Sequence[UpdateAgentRequestToolsItemParams]] = OMIT,
        tasks: typing.Optional[typing.Sequence[TaskParams]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        languages: typing.Optional[typing.Sequence[LanguageCode]] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[UpdateAgentRequestConfigurationEndpointParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentsUpdateResponse]:
        """
        Updates an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to update.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[UpdateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        background_noise_level : typing.Optional[float]
            The background noise level of the agent.

        background_noise : typing.Optional[UpdateAgentRequestBackgroundNoise]
            The background noise type. Can be "office", "call-center", "coffee-shop", or null.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValueParams]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[UpdateAgentRequestToolsItemParams]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[TaskParams]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        languages : typing.Optional[typing.Sequence[LanguageCode]]
            Array of ISO 639-1 language codes that the agent should be able to recognize

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[UpdateAgentRequestConfigurationEndpointParams]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentsUpdateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agents/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            params={
                "project": project,
            },
            json={
                "name": name,
                "phone_number": phone_number,
                "timezone": timezone,
                "voice_id": voice_id,
                "audio_format": audio_format,
                "audio_speed": audio_speed,
                "background_noise_level": background_noise_level,
                "background_noise": background_noise,
                "welcome_message": welcome_message,
                "system_prompt": system_prompt,
                "template_variables": convert_and_respect_annotation_metadata(
                    object_=template_variables,
                    annotation=typing.Dict[str, UpdateAgentRequestTemplateVariablesValueParams],
                    direction="write",
                ),
                "tools": convert_and_respect_annotation_metadata(
                    object_=tools, annotation=typing.Sequence[UpdateAgentRequestToolsItemParams], direction="write"
                ),
                "tasks": convert_and_respect_annotation_metadata(
                    object_=tasks, annotation=typing.Sequence[TaskParams], direction="write"
                ),
                "no_input_poke_sec": no_input_poke_sec,
                "no_input_poke_text": no_input_poke_text,
                "no_input_end_conversation_sec": no_input_end_conversation_sec,
                "languages": languages,
                "boosted_keywords": boosted_keywords,
                "configuration_endpoint": convert_and_respect_annotation_metadata(
                    object_=configuration_endpoint,
                    annotation=typing.Optional[UpdateAgentRequestConfigurationEndpointParams],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentsUpdateResponse,
                    parse_obj_as(
                        type_=AgentsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.tool_parameter_item_type import ToolParameterItemType
from ..types.tool_parameter_location import ToolParameterLocation
from ..types.tool_parameter_type import ToolParameterType


class ToolParameterParams(typing_extensions.TypedDict):
    type: ToolParameterType
    """
    The parameter type.
    """

    item_type: typing_extensions.NotRequired[ToolParameterItemType]
    """
    Required only when type is "array". The type of items in the array.
    """

    name: str
    """
    The parameter name.
    """

    description: str
    """
    Description of the parameter.
    """

    is_required: bool
    """
    Whether the parameter is required.
    """

    location: typing_extensions.NotRequired[ToolParameterLocation]
    """
    Only applicable for `custom_webhook` tools. Specifies where the parameter should be sent in the webhook request.
    - For GET webhooks: defaults to `"query_string"` and `"request_body"` is not allowed.
    - For POST webhooks: required, can be either `"request_body"` or `"query_string"`.
    - Not allowed for `custom_websocket` or `built_in_transfer_to_phone_number` tools.
    When updating a tool's type or endpoint_method, all parameters must include explicit `location` values.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

ConversationEvaluationResultResult = typing.Union[
    typing.Literal["successful", "unsuccessful", "undecided", "error"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.agent_audio_format import AgentAudioFormat
from ..types.agent_background_noise import AgentBackgroundNoise
from ..types.language_code import LanguageCode
from .agent_configuration_endpoint import AgentConfigurationEndpointParams
from .agent_project import AgentProjectParams
from .agent_template_variables_value import AgentTemplateVariablesValueParams
from .agent_tools_item import AgentToolsItemParams
from .task import TaskParams


class AgentParams(typing_extensions.TypedDict):
    id: str
    """
    The ID of the agent.
    """

    name: str
    """
    The name of the agent.
    """

    phone_number: typing.Optional[str]
    """
    The phone number that the agent uses to accept and initiate phone calls. `null` if the agent is not associated with a phone number, in which can the agent can be used via WebSockets.
    """

    project: AgentProjectParams
    """
    The project the agent belongs to.
    """

    timezone: str
    """
    The timezone of the agent. Used to format system variables like `{{system_time}}`.
    """

    voice_id: str
    """
    The voice ID of the agent.
    """

    audio_format: AgentAudioFormat
    """
    The audio format of the agent. If the agent has a phone number, the audio format will be `mulaw_8000`.
    """

    audio_speed: float
    """
    The audio speed of the agent. Must be a multiple of 0.1.
    """

    background_noise_level: float
    """
    The background noise level of the agent. Must be between 0 and 1.
    """

    background_noise: typing.Optional[AgentBackgroundNoise]
    """
    The background noise type. Can be "office", "call-center", "coffee-shop", or null.
    """

    welcome_message: typing.Optional[str]
    """
    Message to play when the conversation starts.
    """

    system_prompt: str
    """
    Instructions for the conversation.
    """

    template_variables: typing.Dict[str, AgentTemplateVariablesValueParams]
    """
    Template variables that the agent can use in the welcome message and the system prompt.
    """

    tools: typing.Sequence[AgentToolsItemParams]
    """
    List of tools available to the agent.
    """

    tasks: typing.Sequence[TaskParams]
    """
    Tasks for the agent to complete during the conversation.
    """

    no_input_poke_sec: typing.Optional[int]
    """
    Number of seconds of silence before sending a poke message. `null` disables the poke message.
    """

    no_input_poke_text: str
    """
    The message to send after the specified silence.
    """

    no_input_end_conversation_sec: int
    """
    Seconds of silence before ending the conversation.
    """

    languages: typing.Sequence[LanguageCode]
    """
    Array of ISO 639-1 language codes that the agent should be able to recognize
    """

    boosted_keywords: typing.Sequence[str]
    """
    These words, or short phrases, will be more accurately recognized by the agent.
    """

    configuration_endpoint: typing.Optional[AgentConfigurationEndpointParams]
    """
    When not `null`, the agent will call this endpoint to get configuration options.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .conversation_evaluation_result import ConversationEvaluationResultParams


class ConversationEvaluationParams(typing_extensions.TypedDict):
    result: ConversationEvaluationResultParams
    """
    The evaluation result.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ToolCallOutputProcessedPayloadToolParams(typing_extensions.TypedDict):
    id: str
    """
    Tool ID
    """

    name: str
    """
    Human-readable tool name
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.extraction_schema import ExtractionSchemaParams


class ExtractionSchemasListResponseParams(typing_extensions.TypedDict):
    extraction_schemas: typing.Sequence[ExtractionSchemaParams]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class ToolCallOutputPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["tool_call_output"]
    tool_call_id: str
    """
    ID of the tool call being responded to
    """

    output: typing.Optional[typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class ConversationCreatedPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["conversation_created"]
    conversation_id: str
    """
    ID of the created conversation
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class IsUserSpeakingPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["is_user_speaking"]
    is_user_speaking: bool
    """
    Whether the user is currently speaking
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .tools_create_response import ToolsCreateResponseParams
    from .tools_delete_response import ToolsDeleteResponseParams
    from .tools_get_response import ToolsGetResponseParams
    from .tools_list_response import ToolsListResponseParams
    from .tools_update_response import ToolsUpdateResponseParams
_dynamic_imports: typing.Dict[str, str] = {
    "ToolsCreateResponseParams": ".tools_create_response",
    "ToolsDeleteResponseParams": ".tools_delete_response",
    "ToolsGetResponseParams": ".tools_get_response",
    "ToolsListResponseParams": ".tools_list_response",
    "ToolsUpdateResponseParams": ".tools_update_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ToolsCreateResponseParams",
    "ToolsDeleteResponseParams",
    "ToolsGetResponseParams",
    "ToolsListResponseParams",
    "ToolsUpdateResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .tool_endpoint_method import ToolEndpointMethod
from .tool_execution_mode import ToolExecutionMode
from .tool_parameter import ToolParameter
from .tool_project import ToolProject
from .tool_type import ToolType


class Tool(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the tool.
    """

    name: str = pydantic.Field()
    """
    The name of the tool.
    """

    description: str = pydantic.Field()
    """
    Description of what the tool does.
    """

    project: ToolProject
    type: ToolType = pydantic.Field()
    """
    The type of tool.
    """

    execution_mode: ToolExecutionMode = pydantic.Field()
    """
    Mode of operation - sync waits for response, async continues without waiting.
    """

    parameters: typing.List[ToolParameter] = pydantic.Field()
    """
    Array of parameter definitions for the tool.
    """

    endpoint_method: typing.Optional[ToolEndpointMethod] = pydantic.Field(default=None)
    """
    HTTP method for webhook tools.
    """

    endpoint_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for webhook tools.
    """

    endpoint_headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Headers for webhook tools.
    """

    endpoint_timeout_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    Timeout in milliseconds for webhook tools.
    """

    tool_call_output_timeout_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    Timeout in milliseconds for WebSocket tool responses.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The E.164 formatted phone number to transfer calls to. Required for built_in_transfer_to_phone_number tools.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ExtractionSchemasUpdateResponseParams(typing_extensions.TypedDict):
    success: bool
    """
    Whether the extraction schema was updated successfully.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class UserFinishedSpeakingPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["user_finished_speaking"]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.conversation_extraction import ConversationExtractionParams


class ConversationsListExtractionsResponseParams(typing_extensions.TypedDict):
    extractions: typing.Sequence[ConversationExtractionParams]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Agent,
        AgentAudioFormat,
        AgentBackgroundNoise,
        AgentConfigurationEndpoint,
        AgentProject,
        AgentTemplateVariablesValue,
        AgentToolsItem,
        AssistantChoseNotToRespondPayload,
        AssistantEndedConversationPayload,
        AudioChunkPayload,
        AudioChunkResponsePayload,
        BadRequestErrorBody,
        BasicError,
        BasicErrorError,
        ConfigPayload,
        ConfigPayloadBackgroundNoise,
        ConfigPayloadInputFormat,
        ConfigPayloadOutputFormat,
        Conversation,
        ConversationAgent,
        ConversationAnalysis,
        ConversationBackgroundNoise,
        ConversationCreatedPayload,
        ConversationEndedBy,
        ConversationEval,
        ConversationEvalPrompt,
        ConversationEvaluation,
        ConversationEvaluationResult,
        ConversationEvaluationResultPrompt,
        ConversationEvaluationResultResult,
        ConversationExtraction,
        ConversationExtractionSchema,
        ConversationItem,
        ConversationItemRole,
        ConversationItemToolCallsItem,
        ConversationItemToolCallsItemTool,
        ConversationProject,
        CreateAgentRequest,
        CreateAgentRequestAudioFormat,
        CreateAgentRequestBackgroundNoise,
        CreateAgentRequestConfigurationEndpoint,
        CreateAgentRequestTemplateVariablesValue,
        CreateAgentRequestToolsItem,
        DtmfPayload,
        Error,
        ErrorPayload,
        ErrorPayloadError,
        ExtractionField,
        ExtractionFieldType,
        ExtractionSchema,
        InputCancelledPayload,
        InputTextPayload,
        IsUserSpeakingPayload,
        LanguageCode,
        OutboundCallConfig,
        OutboundCallConfigToolsItem,
        Project,
        ProjectDefaultAgent,
        ReadyToStartConversationPayload,
        SetExternalIdPayload,
        SetTwilioCallSidPayload,
        Task,
        Tool,
        ToolCallInterruptedPayload,
        ToolCallOutputPayload,
        ToolCallOutputProcessedPayload,
        ToolCallOutputProcessedPayloadTool,
        ToolCallPayload,
        ToolEndpointMethod,
        ToolExecutionMode,
        ToolParameter,
        ToolParameterItemType,
        ToolParameterLocation,
        ToolParameterType,
        ToolProject,
        ToolType,
        UpdateSystemPromptPayload,
        UserFinishedSpeakingPayload,
        UserStartedSpeakingPayload,
        ValidationError,
        ValidationErrorError,
        Voice,
    )
    from .errors import (
        BadRequestError,
        ConflictError,
        ForbiddenError,
        GatewayTimeoutError,
        InternalServerError,
        NotFoundError,
        UnauthorizedError,
    )
    from . import agents, conversations, extraction_schemas, projects, tools, voices
    from .agents import (
        AgentsCreateResponse,
        AgentsCreateResponseParams,
        AgentsDeleteResponse,
        AgentsDeleteResponseParams,
        AgentsGetResponse,
        AgentsGetResponseParams,
        AgentsListResponse,
        AgentsListResponseParams,
        AgentsUpdateResponse,
        AgentsUpdateResponseParams,
        AgentsUpsertResponse,
        AgentsUpsertResponseParams,
        UpdateAgentRequestAudioFormat,
        UpdateAgentRequestBackgroundNoise,
        UpdateAgentRequestConfigurationEndpoint,
        UpdateAgentRequestConfigurationEndpointParams,
        UpdateAgentRequestTemplateVariablesValue,
        UpdateAgentRequestTemplateVariablesValueParams,
        UpdateAgentRequestToolsItem,
        UpdateAgentRequestToolsItemParams,
    )
    from .client import AsyncPhonic, Phonic
    from .conversations import (
        ConversationsCancelResponse,
        ConversationsCancelResponseParams,
        ConversationsExtractDataResponse,
        ConversationsExtractDataResponseParams,
        ConversationsGetAnalysisResponse,
        ConversationsGetAnalysisResponseParams,
        ConversationsGetResponse,
        ConversationsGetResponseParams,
        ConversationsListEvaluationsResponse,
        ConversationsListEvaluationsResponseParams,
        ConversationsListExtractionsResponse,
        ConversationsListExtractionsResponseParams,
        ConversationsListResponse,
        ConversationsListResponseConversation,
        ConversationsListResponseConversationParams,
        ConversationsListResponseConversations,
        ConversationsListResponseConversationsParams,
        ConversationsListResponseParams,
        ConversationsOutboundCallResponse,
        ConversationsOutboundCallResponseParams,
        ConversationsSipOutboundCallResponse,
        ConversationsSipOutboundCallResponseParams,
    )
    from .environment import PhonicEnvironment
    from .extraction_schemas import (
        ExtractionSchemasCreateResponse,
        ExtractionSchemasCreateResponseParams,
        ExtractionSchemasDeleteResponse,
        ExtractionSchemasDeleteResponseParams,
        ExtractionSchemasGetResponse,
        ExtractionSchemasGetResponseParams,
        ExtractionSchemasListResponse,
        ExtractionSchemasListResponseParams,
        ExtractionSchemasUpdateResponse,
        ExtractionSchemasUpdateResponseParams,
    )
    from .projects import (
        ProjectsCreateEvalPromptResponse,
        ProjectsCreateEvalPromptResponseParams,
        ProjectsCreateResponse,
        ProjectsCreateResponseParams,
        ProjectsDeleteResponse,
        ProjectsDeleteResponseParams,
        ProjectsGetResponse,
        ProjectsGetResponseParams,
        ProjectsListEvalPromptsResponse,
        ProjectsListEvalPromptsResponseParams,
        ProjectsListResponse,
        ProjectsListResponseParams,
        ProjectsUpdateResponse,
        ProjectsUpdateResponseParams,
    )
    from .requests import (
        AgentConfigurationEndpointParams,
        AgentParams,
        AgentProjectParams,
        AgentTemplateVariablesValueParams,
        AgentToolsItemParams,
        AssistantChoseNotToRespondPayloadParams,
        AssistantEndedConversationPayloadParams,
        AudioChunkPayloadParams,
        AudioChunkResponsePayloadParams,
        BadRequestErrorBodyParams,
        BasicErrorErrorParams,
        BasicErrorParams,
        ConfigPayloadParams,
        ConversationAgentParams,
        ConversationAnalysisParams,
        ConversationCreatedPayloadParams,
        ConversationEvalPromptParams,
        ConversationEvaluationParams,
        ConversationEvaluationResultParams,
        ConversationEvaluationResultPromptParams,
        ConversationExtractionParams,
        ConversationExtractionSchemaParams,
        ConversationItemParams,
        ConversationItemToolCallsItemParams,
        ConversationItemToolCallsItemToolParams,
        ConversationParams,
        ConversationProjectParams,
        CreateAgentRequestConfigurationEndpointParams,
        CreateAgentRequestParams,
        CreateAgentRequestTemplateVariablesValueParams,
        CreateAgentRequestToolsItemParams,
        DtmfPayloadParams,
        ErrorParams,
        ErrorPayloadErrorParams,
        ErrorPayloadParams,
        ExtractionFieldParams,
        ExtractionSchemaParams,
        InputCancelledPayloadParams,
        InputTextPayloadParams,
        IsUserSpeakingPayloadParams,
        OutboundCallConfigParams,
        OutboundCallConfigToolsItemParams,
        ProjectDefaultAgentParams,
        ProjectParams,
        ReadyToStartConversationPayloadParams,
        SetExternalIdPayloadParams,
        SetTwilioCallSidPayloadParams,
        TaskParams,
        ToolCallInterruptedPayloadParams,
        ToolCallOutputPayloadParams,
        ToolCallOutputProcessedPayloadParams,
        ToolCallOutputProcessedPayloadToolParams,
        ToolCallPayloadParams,
        ToolParameterParams,
        ToolParams,
        ToolProjectParams,
        UpdateSystemPromptPayloadParams,
        UserFinishedSpeakingPayloadParams,
        UserStartedSpeakingPayloadParams,
        ValidationErrorErrorParams,
        ValidationErrorParams,
        VoiceParams,
    )
    from .tools import (
        CreateToolRequestEndpointMethod,
        CreateToolRequestExecutionMode,
        CreateToolRequestType,
        ToolsCreateResponse,
        ToolsCreateResponseParams,
        ToolsDeleteResponse,
        ToolsDeleteResponseParams,
        ToolsGetResponse,
        ToolsGetResponseParams,
        ToolsListResponse,
        ToolsListResponseParams,
        ToolsUpdateResponse,
        ToolsUpdateResponseParams,
        UpdateToolRequestEndpointMethod,
        UpdateToolRequestExecutionMode,
        UpdateToolRequestType,
    )
    from .version import __version__
    from .voices import VoicesGetResponse, VoicesGetResponseParams, VoicesListResponse, VoicesListResponseParams
_dynamic_imports: typing.Dict[str, str] = {
    "Agent": ".types",
    "AgentAudioFormat": ".types",
    "AgentBackgroundNoise": ".types",
    "AgentConfigurationEndpoint": ".types",
    "AgentConfigurationEndpointParams": ".requests",
    "AgentParams": ".requests",
    "AgentProject": ".types",
    "AgentProjectParams": ".requests",
    "AgentTemplateVariablesValue": ".types",
    "AgentTemplateVariablesValueParams": ".requests",
    "AgentToolsItem": ".types",
    "AgentToolsItemParams": ".requests",
    "AgentsCreateResponse": ".agents",
    "AgentsCreateResponseParams": ".agents",
    "AgentsDeleteResponse": ".agents",
    "AgentsDeleteResponseParams": ".agents",
    "AgentsGetResponse": ".agents",
    "AgentsGetResponseParams": ".agents",
    "AgentsListResponse": ".agents",
    "AgentsListResponseParams": ".agents",
    "AgentsUpdateResponse": ".agents",
    "AgentsUpdateResponseParams": ".agents",
    "AgentsUpsertResponse": ".agents",
    "AgentsUpsertResponseParams": ".agents",
    "AssistantChoseNotToRespondPayload": ".types",
    "AssistantChoseNotToRespondPayloadParams": ".requests",
    "AssistantEndedConversationPayload": ".types",
    "AssistantEndedConversationPayloadParams": ".requests",
    "AsyncPhonic": ".client",
    "AudioChunkPayload": ".types",
    "AudioChunkPayloadParams": ".requests",
    "AudioChunkResponsePayload": ".types",
    "AudioChunkResponsePayloadParams": ".requests",
    "BadRequestError": ".errors",
    "BadRequestErrorBody": ".types",
    "BadRequestErrorBodyParams": ".requests",
    "BasicError": ".types",
    "BasicErrorError": ".types",
    "BasicErrorErrorParams": ".requests",
    "BasicErrorParams": ".requests",
    "ConfigPayload": ".types",
    "ConfigPayloadBackgroundNoise": ".types",
    "ConfigPayloadInputFormat": ".types",
    "ConfigPayloadOutputFormat": ".types",
    "ConfigPayloadParams": ".requests",
    "ConflictError": ".errors",
    "Conversation": ".types",
    "ConversationAgent": ".types",
    "ConversationAgentParams": ".requests",
    "ConversationAnalysis": ".types",
    "ConversationAnalysisParams": ".requests",
    "ConversationBackgroundNoise": ".types",
    "ConversationCreatedPayload": ".types",
    "ConversationCreatedPayloadParams": ".requests",
    "ConversationEndedBy": ".types",
    "ConversationEval": ".types",
    "ConversationEvalPrompt": ".types",
    "ConversationEvalPromptParams": ".requests",
    "ConversationEvaluation": ".types",
    "ConversationEvaluationParams": ".requests",
    "ConversationEvaluationResult": ".types",
    "ConversationEvaluationResultParams": ".requests",
    "ConversationEvaluationResultPrompt": ".types",
    "ConversationEvaluationResultPromptParams": ".requests",
    "ConversationEvaluationResultResult": ".types",
    "ConversationExtraction": ".types",
    "ConversationExtractionParams": ".requests",
    "ConversationExtractionSchema": ".types",
    "ConversationExtractionSchemaParams": ".requests",
    "ConversationItem": ".types",
    "ConversationItemParams": ".requests",
    "ConversationItemRole": ".types",
    "ConversationItemToolCallsItem": ".types",
    "ConversationItemToolCallsItemParams": ".requests",
    "ConversationItemToolCallsItemTool": ".types",
    "ConversationItemToolCallsItemToolParams": ".requests",
    "ConversationParams": ".requests",
    "ConversationProject": ".types",
    "ConversationProjectParams": ".requests",
    "ConversationsCancelResponse": ".conversations",
    "ConversationsCancelResponseParams": ".conversations",
    "ConversationsExtractDataResponse": ".conversations",
    "ConversationsExtractDataResponseParams": ".conversations",
    "ConversationsGetAnalysisResponse": ".conversations",
    "ConversationsGetAnalysisResponseParams": ".conversations",
    "ConversationsGetResponse": ".conversations",
    "ConversationsGetResponseParams": ".conversations",
    "ConversationsListEvaluationsResponse": ".conversations",
    "ConversationsListEvaluationsResponseParams": ".conversations",
    "ConversationsListExtractionsResponse": ".conversations",
    "ConversationsListExtractionsResponseParams": ".conversations",
    "ConversationsListResponse": ".conversations",
    "ConversationsListResponseConversation": ".conversations",
    "ConversationsListResponseConversationParams": ".conversations",
    "ConversationsListResponseConversations": ".conversations",
    "ConversationsListResponseConversationsParams": ".conversations",
    "ConversationsListResponseParams": ".conversations",
    "ConversationsOutboundCallResponse": ".conversations",
    "ConversationsOutboundCallResponseParams": ".conversations",
    "ConversationsSipOutboundCallResponse": ".conversations",
    "ConversationsSipOutboundCallResponseParams": ".conversations",
    "CreateAgentRequest": ".types",
    "CreateAgentRequestAudioFormat": ".types",
    "CreateAgentRequestBackgroundNoise": ".types",
    "CreateAgentRequestConfigurationEndpoint": ".types",
    "CreateAgentRequestConfigurationEndpointParams": ".requests",
    "CreateAgentRequestParams": ".requests",
    "CreateAgentRequestTemplateVariablesValue": ".types",
    "CreateAgentRequestTemplateVariablesValueParams": ".requests",
    "CreateAgentRequestToolsItem": ".types",
    "CreateAgentRequestToolsItemParams": ".requests",
    "CreateToolRequestEndpointMethod": ".tools",
    "CreateToolRequestExecutionMode": ".tools",
    "CreateToolRequestType": ".tools",
    "DtmfPayload": ".types",
    "DtmfPayloadParams": ".requests",
    "Error": ".types",
    "ErrorParams": ".requests",
    "ErrorPayload": ".types",
    "ErrorPayloadError": ".types",
    "ErrorPayloadErrorParams": ".requests",
    "ErrorPayloadParams": ".requests",
    "ExtractionField": ".types",
    "ExtractionFieldParams": ".requests",
    "ExtractionFieldType": ".types",
    "ExtractionSchema": ".types",
    "ExtractionSchemaParams": ".requests",
    "ExtractionSchemasCreateResponse": ".extraction_schemas",
    "ExtractionSchemasCreateResponseParams": ".extraction_schemas",
    "ExtractionSchemasDeleteResponse": ".extraction_schemas",
    "ExtractionSchemasDeleteResponseParams": ".extraction_schemas",
    "ExtractionSchemasGetResponse": ".extraction_schemas",
    "ExtractionSchemasGetResponseParams": ".extraction_schemas",
    "ExtractionSchemasListResponse": ".extraction_schemas",
    "ExtractionSchemasListResponseParams": ".extraction_schemas",
    "ExtractionSchemasUpdateResponse": ".extraction_schemas",
    "ExtractionSchemasUpdateResponseParams": ".extraction_schemas",
    "ForbiddenError": ".errors",
    "GatewayTimeoutError": ".errors",
    "InputCancelledPayload": ".types",
    "InputCancelledPayloadParams": ".requests",
    "InputTextPayload": ".types",
    "InputTextPayloadParams": ".requests",
    "InternalServerError": ".errors",
    "IsUserSpeakingPayload": ".types",
    "IsUserSpeakingPayloadParams": ".requests",
    "LanguageCode": ".types",
    "NotFoundError": ".errors",
    "OutboundCallConfig": ".types",
    "OutboundCallConfigParams": ".requests",
    "OutboundCallConfigToolsItem": ".types",
    "OutboundCallConfigToolsItemParams": ".requests",
    "Phonic": ".client",
    "PhonicEnvironment": ".environment",
    "Project": ".types",
    "ProjectDefaultAgent": ".types",
    "ProjectDefaultAgentParams": ".requests",
    "ProjectParams": ".requests",
    "ProjectsCreateEvalPromptResponse": ".projects",
    "ProjectsCreateEvalPromptResponseParams": ".projects",
    "ProjectsCreateResponse": ".projects",
    "ProjectsCreateResponseParams": ".projects",
    "ProjectsDeleteResponse": ".projects",
    "ProjectsDeleteResponseParams": ".projects",
    "ProjectsGetResponse": ".projects",
    "ProjectsGetResponseParams": ".projects",
    "ProjectsListEvalPromptsResponse": ".projects",
    "ProjectsListEvalPromptsResponseParams": ".projects",
    "ProjectsListResponse": ".projects",
    "ProjectsListResponseParams": ".projects",
    "ProjectsUpdateResponse": ".projects",
    "ProjectsUpdateResponseParams": ".projects",
    "ReadyToStartConversationPayload": ".types",
    "ReadyToStartConversationPayloadParams": ".requests",
    "SetExternalIdPayload": ".types",
    "SetExternalIdPayloadParams": ".requests",
    "SetTwilioCallSidPayload": ".types",
    "SetTwilioCallSidPayloadParams": ".requests",
    "Task": ".types",
    "TaskParams": ".requests",
    "Tool": ".types",
    "ToolCallInterruptedPayload": ".types",
    "ToolCallInterruptedPayloadParams": ".requests",
    "ToolCallOutputPayload": ".types",
    "ToolCallOutputPayloadParams": ".requests",
    "ToolCallOutputProcessedPayload": ".types",
    "ToolCallOutputProcessedPayloadParams": ".requests",
    "ToolCallOutputProcessedPayloadTool": ".types",
    "ToolCallOutputProcessedPayloadToolParams": ".requests",
    "ToolCallPayload": ".types",
    "ToolCallPayloadParams": ".requests",
    "ToolEndpointMethod": ".types",
    "ToolExecutionMode": ".types",
    "ToolParameter": ".types",
    "ToolParameterItemType": ".types",
    "ToolParameterLocation": ".types",
    "ToolParameterParams": ".requests",
    "ToolParameterType": ".types",
    "ToolParams": ".requests",
    "ToolProject": ".types",
    "ToolProjectParams": ".requests",
    "ToolType": ".types",
    "ToolsCreateResponse": ".tools",
    "ToolsCreateResponseParams": ".tools",
    "ToolsDeleteResponse": ".tools",
    "ToolsDeleteResponseParams": ".tools",
    "ToolsGetResponse": ".tools",
    "ToolsGetResponseParams": ".tools",
    "ToolsListResponse": ".tools",
    "ToolsListResponseParams": ".tools",
    "ToolsUpdateResponse": ".tools",
    "ToolsUpdateResponseParams": ".tools",
    "UnauthorizedError": ".errors",
    "UpdateAgentRequestAudioFormat": ".agents",
    "UpdateAgentRequestBackgroundNoise": ".agents",
    "UpdateAgentRequestConfigurationEndpoint": ".agents",
    "UpdateAgentRequestConfigurationEndpointParams": ".agents",
    "UpdateAgentRequestTemplateVariablesValue": ".agents",
    "UpdateAgentRequestTemplateVariablesValueParams": ".agents",
    "UpdateAgentRequestToolsItem": ".agents",
    "UpdateAgentRequestToolsItemParams": ".agents",
    "UpdateSystemPromptPayload": ".types",
    "UpdateSystemPromptPayloadParams": ".requests",
    "UpdateToolRequestEndpointMethod": ".tools",
    "UpdateToolRequestExecutionMode": ".tools",
    "UpdateToolRequestType": ".tools",
    "UserFinishedSpeakingPayload": ".types",
    "UserFinishedSpeakingPayloadParams": ".requests",
    "UserStartedSpeakingPayload": ".types",
    "UserStartedSpeakingPayloadParams": ".requests",
    "ValidationError": ".types",
    "ValidationErrorError": ".types",
    "ValidationErrorErrorParams": ".requests",
    "ValidationErrorParams": ".requests",
    "Voice": ".types",
    "VoiceParams": ".requests",
    "VoicesGetResponse": ".voices",
    "VoicesGetResponseParams": ".voices",
    "VoicesListResponse": ".voices",
    "VoicesListResponseParams": ".voices",
    "__version__": ".version",
    "agents": ".agents",
    "conversations": ".conversations",
    "extraction_schemas": ".extraction_schemas",
    "projects": ".projects",
    "tools": ".tools",
    "voices": ".voices",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Agent",
    "AgentAudioFormat",
    "AgentBackgroundNoise",
    "AgentConfigurationEndpoint",
    "AgentConfigurationEndpointParams",
    "AgentParams",
    "AgentProject",
    "AgentProjectParams",
    "AgentTemplateVariablesValue",
    "AgentTemplateVariablesValueParams",
    "AgentToolsItem",
    "AgentToolsItemParams",
    "AgentsCreateResponse",
    "AgentsCreateResponseParams",
    "AgentsDeleteResponse",
    "AgentsDeleteResponseParams",
    "AgentsGetResponse",
    "AgentsGetResponseParams",
    "AgentsListResponse",
    "AgentsListResponseParams",
    "AgentsUpdateResponse",
    "AgentsUpdateResponseParams",
    "AgentsUpsertResponse",
    "AgentsUpsertResponseParams",
    "AssistantChoseNotToRespondPayload",
    "AssistantChoseNotToRespondPayloadParams",
    "AssistantEndedConversationPayload",
    "AssistantEndedConversationPayloadParams",
    "AsyncPhonic",
    "AudioChunkPayload",
    "AudioChunkPayloadParams",
    "AudioChunkResponsePayload",
    "AudioChunkResponsePayloadParams",
    "BadRequestError",
    "BadRequestErrorBody",
    "BadRequestErrorBodyParams",
    "BasicError",
    "BasicErrorError",
    "BasicErrorErrorParams",
    "BasicErrorParams",
    "ConfigPayload",
    "ConfigPayloadBackgroundNoise",
    "ConfigPayloadInputFormat",
    "ConfigPayloadOutputFormat",
    "ConfigPayloadParams",
    "ConflictError",
    "Conversation",
    "ConversationAgent",
    "ConversationAgentParams",
    "ConversationAnalysis",
    "ConversationAnalysisParams",
    "ConversationBackgroundNoise",
    "ConversationCreatedPayload",
    "ConversationCreatedPayloadParams",
    "ConversationEndedBy",
    "ConversationEval",
    "ConversationEvalPrompt",
    "ConversationEvalPromptParams",
    "ConversationEvaluation",
    "ConversationEvaluationParams",
    "ConversationEvaluationResult",
    "ConversationEvaluationResultParams",
    "ConversationEvaluationResultPrompt",
    "ConversationEvaluationResultPromptParams",
    "ConversationEvaluationResultResult",
    "ConversationExtraction",
    "ConversationExtractionParams",
    "ConversationExtractionSchema",
    "ConversationExtractionSchemaParams",
    "ConversationItem",
    "ConversationItemParams",
    "ConversationItemRole",
    "ConversationItemToolCallsItem",
    "ConversationItemToolCallsItemParams",
    "ConversationItemToolCallsItemTool",
    "ConversationItemToolCallsItemToolParams",
    "ConversationParams",
    "ConversationProject",
    "ConversationProjectParams",
    "ConversationsCancelResponse",
    "ConversationsCancelResponseParams",
    "ConversationsExtractDataResponse",
    "ConversationsExtractDataResponseParams",
    "ConversationsGetAnalysisResponse",
    "ConversationsGetAnalysisResponseParams",
    "ConversationsGetResponse",
    "ConversationsGetResponseParams",
    "ConversationsListEvaluationsResponse",
    "ConversationsListEvaluationsResponseParams",
    "ConversationsListExtractionsResponse",
    "ConversationsListExtractionsResponseParams",
    "ConversationsListResponse",
    "ConversationsListResponseConversation",
    "ConversationsListResponseConversationParams",
    "ConversationsListResponseConversations",
    "ConversationsListResponseConversationsParams",
    "ConversationsListResponseParams",
    "ConversationsOutboundCallResponse",
    "ConversationsOutboundCallResponseParams",
    "ConversationsSipOutboundCallResponse",
    "ConversationsSipOutboundCallResponseParams",
    "CreateAgentRequest",
    "CreateAgentRequestAudioFormat",
    "CreateAgentRequestBackgroundNoise",
    "CreateAgentRequestConfigurationEndpoint",
    "CreateAgentRequestConfigurationEndpointParams",
    "CreateAgentRequestParams",
    "CreateAgentRequestTemplateVariablesValue",
    "CreateAgentRequestTemplateVariablesValueParams",
    "CreateAgentRequestToolsItem",
    "CreateAgentRequestToolsItemParams",
    "CreateToolRequestEndpointMethod",
    "CreateToolRequestExecutionMode",
    "CreateToolRequestType",
    "DtmfPayload",
    "DtmfPayloadParams",
    "Error",
    "ErrorParams",
    "ErrorPayload",
    "ErrorPayloadError",
    "ErrorPayloadErrorParams",
    "ErrorPayloadParams",
    "ExtractionField",
    "ExtractionFieldParams",
    "ExtractionFieldType",
    "ExtractionSchema",
    "ExtractionSchemaParams",
    "ExtractionSchemasCreateResponse",
    "ExtractionSchemasCreateResponseParams",
    "ExtractionSchemasDeleteResponse",
    "ExtractionSchemasDeleteResponseParams",
    "ExtractionSchemasGetResponse",
    "ExtractionSchemasGetResponseParams",
    "ExtractionSchemasListResponse",
    "ExtractionSchemasListResponseParams",
    "ExtractionSchemasUpdateResponse",
    "ExtractionSchemasUpdateResponseParams",
    "ForbiddenError",
    "GatewayTimeoutError",
    "InputCancelledPayload",
    "InputCancelledPayloadParams",
    "InputTextPayload",
    "InputTextPayloadParams",
    "InternalServerError",
    "IsUserSpeakingPayload",
    "IsUserSpeakingPayloadParams",
    "LanguageCode",
    "NotFoundError",
    "OutboundCallConfig",
    "OutboundCallConfigParams",
    "OutboundCallConfigToolsItem",
    "OutboundCallConfigToolsItemParams",
    "Phonic",
    "PhonicEnvironment",
    "Project",
    "ProjectDefaultAgent",
    "ProjectDefaultAgentParams",
    "ProjectParams",
    "ProjectsCreateEvalPromptResponse",
    "ProjectsCreateEvalPromptResponseParams",
    "ProjectsCreateResponse",
    "ProjectsCreateResponseParams",
    "ProjectsDeleteResponse",
    "ProjectsDeleteResponseParams",
    "ProjectsGetResponse",
    "ProjectsGetResponseParams",
    "ProjectsListEvalPromptsResponse",
    "ProjectsListEvalPromptsResponseParams",
    "ProjectsListResponse",
    "ProjectsListResponseParams",
    "ProjectsUpdateResponse",
    "ProjectsUpdateResponseParams",
    "ReadyToStartConversationPayload",
    "ReadyToStartConversationPayloadParams",
    "SetExternalIdPayload",
    "SetExternalIdPayloadParams",
    "SetTwilioCallSidPayload",
    "SetTwilioCallSidPayloadParams",
    "Task",
    "TaskParams",
    "Tool",
    "ToolCallInterruptedPayload",
    "ToolCallInterruptedPayloadParams",
    "ToolCallOutputPayload",
    "ToolCallOutputPayloadParams",
    "ToolCallOutputProcessedPayload",
    "ToolCallOutputProcessedPayloadParams",
    "ToolCallOutputProcessedPayloadTool",
    "ToolCallOutputProcessedPayloadToolParams",
    "ToolCallPayload",
    "ToolCallPayloadParams",
    "ToolEndpointMethod",
    "ToolExecutionMode",
    "ToolParameter",
    "ToolParameterItemType",
    "ToolParameterLocation",
    "ToolParameterParams",
    "ToolParameterType",
    "ToolParams",
    "ToolProject",
    "ToolProjectParams",
    "ToolType",
    "ToolsCreateResponse",
    "ToolsCreateResponseParams",
    "ToolsDeleteResponse",
    "ToolsDeleteResponseParams",
    "ToolsGetResponse",
    "ToolsGetResponseParams",
    "ToolsListResponse",
    "ToolsListResponseParams",
    "ToolsUpdateResponse",
    "ToolsUpdateResponseParams",
    "UnauthorizedError",
    "UpdateAgentRequestAudioFormat",
    "UpdateAgentRequestBackgroundNoise",
    "UpdateAgentRequestConfigurationEndpoint",
    "UpdateAgentRequestConfigurationEndpointParams",
    "UpdateAgentRequestTemplateVariablesValue",
    "UpdateAgentRequestTemplateVariablesValueParams",
    "UpdateAgentRequestToolsItem",
    "UpdateAgentRequestToolsItemParams",
    "UpdateSystemPromptPayload",
    "UpdateSystemPromptPayloadParams",
    "UpdateToolRequestEndpointMethod",
    "UpdateToolRequestExecutionMode",
    "UpdateToolRequestType",
    "UserFinishedSpeakingPayload",
    "UserFinishedSpeakingPayloadParams",
    "UserStartedSpeakingPayload",
    "UserStartedSpeakingPayloadParams",
    "ValidationError",
    "ValidationErrorError",
    "ValidationErrorErrorParams",
    "ValidationErrorParams",
    "Voice",
    "VoiceParams",
    "VoicesGetResponse",
    "VoicesGetResponseParams",
    "VoicesListResponse",
    "VoicesListResponseParams",
    "__version__",
    "agents",
    "conversations",
    "extraction_schemas",
    "projects",
    "tools",
    "voices",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.tool import ToolParams


class ToolsListResponseParams(typing_extensions.TypedDict):
    tools: typing.Sequence[ToolParams]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class AgentProjectParams(typing_extensions.TypedDict):
    """
    The project the agent belongs to.
    """

    id: str
    name: str



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class SetExternalIdPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["set_external_id"]
    external_id: str
    """
    External ID to associate with conversation
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.create_agent_request_audio_format import CreateAgentRequestAudioFormat
from ..types.create_agent_request_background_noise import CreateAgentRequestBackgroundNoise
from ..types.language_code import LanguageCode
from .create_agent_request_configuration_endpoint import CreateAgentRequestConfigurationEndpointParams
from .create_agent_request_template_variables_value import CreateAgentRequestTemplateVariablesValueParams
from .create_agent_request_tools_item import CreateAgentRequestToolsItemParams
from .task import TaskParams


class CreateAgentRequestParams(typing_extensions.TypedDict):
    name: str
    """
    The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.
    """

    phone_number: typing_extensions.NotRequired[typing.Optional[typing.Literal["assign-automatically"]]]
    timezone: typing_extensions.NotRequired[str]
    """
    The timezone of the agent. Used to format system variables like `{{system_time}}`.
    """

    voice_id: typing_extensions.NotRequired[str]
    """
    The voice ID to use.
    """

    audio_format: typing_extensions.NotRequired[CreateAgentRequestAudioFormat]
    """
    The audio format of the agent.
    """

    audio_speed: typing_extensions.NotRequired[float]
    """
    The audio speed of the agent.
    """

    background_noise_level: typing_extensions.NotRequired[float]
    """
    The background noise level of the agent.
    """

    background_noise: typing_extensions.NotRequired[typing.Optional[CreateAgentRequestBackgroundNoise]]
    """
    The background noise type. Can be "office", "call-center", "coffee-shop", or null.
    """

    welcome_message: typing_extensions.NotRequired[str]
    """
    Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.
    """

    system_prompt: typing_extensions.NotRequired[str]
    """
    Instructions for the conversation. Can contain template variables like `{{subject}}`.
    """

    template_variables: typing_extensions.NotRequired[typing.Dict[str, CreateAgentRequestTemplateVariablesValueParams]]
    """
    Variables that can be used in the welcome message and the system prompt.
    """

    tools: typing_extensions.NotRequired[typing.Sequence[CreateAgentRequestToolsItemParams]]
    """
    Array of built-in or custom tool names to use.
    """

    tasks: typing_extensions.NotRequired[typing.Sequence[TaskParams]]
    """
    Array of task objects with `name` and `description` fields.
    """

    no_input_poke_sec: typing_extensions.NotRequired[typing.Optional[int]]
    """
    Number of seconds of silence before sending a poke message. `null` disables the poke message.
    """

    no_input_poke_text: typing_extensions.NotRequired[str]
    """
    The message to send after the specified silence.
    """

    no_input_end_conversation_sec: typing_extensions.NotRequired[int]
    """
    Seconds of silence before ending the conversation.
    """

    languages: typing_extensions.NotRequired[typing.Sequence[LanguageCode]]
    """
    Array of ISO 639-1 language codes that the agent should be able to recognize
    """

    boosted_keywords: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    These words, or short phrases, will be more accurately recognized by the agent.
    """

    configuration_endpoint: typing_extensions.NotRequired[
        typing.Optional[CreateAgentRequestConfigurationEndpointParams]
    ]
    """
    When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .create_agent_request_audio_format import CreateAgentRequestAudioFormat
from .create_agent_request_background_noise import CreateAgentRequestBackgroundNoise
from .create_agent_request_configuration_endpoint import CreateAgentRequestConfigurationEndpoint
from .create_agent_request_template_variables_value import CreateAgentRequestTemplateVariablesValue
from .create_agent_request_tools_item import CreateAgentRequestToolsItem
from .language_code import LanguageCode
from .task import Task


class CreateAgentRequest(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.
    """

    phone_number: typing.Optional[typing.Literal["assign-automatically"]] = None
    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timezone of the agent. Used to format system variables like `{{system_time}}`.
    """

    voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The voice ID to use.
    """

    audio_format: typing.Optional[CreateAgentRequestAudioFormat] = pydantic.Field(default=None)
    """
    The audio format of the agent.
    """

    audio_speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    The audio speed of the agent.
    """

    background_noise_level: typing.Optional[float] = pydantic.Field(default=None)
    """
    The background noise level of the agent.
    """

    background_noise: typing.Optional[CreateAgentRequestBackgroundNoise] = pydantic.Field(default=None)
    """
    The background noise type. Can be "office", "call-center", "coffee-shop", or null.
    """

    welcome_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.
    """

    system_prompt: typing.Optional[str] = pydantic.Field(default=None)
    """
    Instructions for the conversation. Can contain template variables like `{{subject}}`.
    """

    template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]] = pydantic.Field(
        default=None
    )
    """
    Variables that can be used in the welcome message and the system prompt.
    """

    tools: typing.Optional[typing.List[CreateAgentRequestToolsItem]] = pydantic.Field(default=None)
    """
    Array of built-in or custom tool names to use.
    """

    tasks: typing.Optional[typing.List[Task]] = pydantic.Field(default=None)
    """
    Array of task objects with `name` and `description` fields.
    """

    no_input_poke_sec: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of seconds of silence before sending a poke message. `null` disables the poke message.
    """

    no_input_poke_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The message to send after the specified silence.
    """

    no_input_end_conversation_sec: typing.Optional[int] = pydantic.Field(default=None)
    """
    Seconds of silence before ending the conversation.
    """

    languages: typing.Optional[typing.List[LanguageCode]] = pydantic.Field(default=None)
    """
    Array of ISO 639-1 language codes that the agent should be able to recognize
    """

    boosted_keywords: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    These words, or short phrases, will be more accurately recognized by the agent.
    """

    configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpoint] = pydantic.Field(default=None)
    """
    When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class UpdateSystemPromptPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["update_system_prompt"]
    system_prompt: str
    """
    New system prompt (cannot contain template variables)
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .project_default_agent import ProjectDefaultAgentParams


class ProjectParams(typing_extensions.TypedDict):
    id: str
    """
    The ID of the project.
    """

    name: str
    """
    The name of the project.
    """

    default_agent: typing.Optional[ProjectDefaultAgentParams]
    """
    The project's default agent.
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import os
import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import PhonicEnvironment

if typing.TYPE_CHECKING:
    from .agents.client import AgentsClient, AsyncAgentsClient
    from .conversations.client import AsyncConversationsClient, ConversationsClient
    from .extraction_schemas.client import AsyncExtractionSchemasClient, ExtractionSchemasClient
    from .projects.client import AsyncProjectsClient, ProjectsClient
    from .tools.client import AsyncToolsClient, ToolsClient
    from .voices.client import AsyncVoicesClient, VoicesClient


class Phonic:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : PhonicEnvironment
        The environment to use for requests from the client. from .environment import PhonicEnvironment



        Defaults to PhonicEnvironment.DEFAULT



    api_key : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phonic import Phonic

    client = Phonic(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: PhonicEnvironment = PhonicEnvironment.DEFAULT,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("PHONIC_API_KEY"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            environment=environment,
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._agents: typing.Optional[AgentsClient] = None
        self._tools: typing.Optional[ToolsClient] = None
        self._extraction_schemas: typing.Optional[ExtractionSchemasClient] = None
        self._voices: typing.Optional[VoicesClient] = None
        self._conversations: typing.Optional[ConversationsClient] = None
        self._projects: typing.Optional[ProjectsClient] = None

    @property
    def agents(self):
        if self._agents is None:
            from .agents.client import AgentsClient  # noqa: E402

            self._agents = AgentsClient(client_wrapper=self._client_wrapper)
        return self._agents

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import ToolsClient  # noqa: E402

            self._tools = ToolsClient(client_wrapper=self._client_wrapper)
        return self._tools

    @property
    def extraction_schemas(self):
        if self._extraction_schemas is None:
            from .extraction_schemas.client import ExtractionSchemasClient  # noqa: E402

            self._extraction_schemas = ExtractionSchemasClient(client_wrapper=self._client_wrapper)
        return self._extraction_schemas

    @property
    def voices(self):
        if self._voices is None:
            from .voices.client import VoicesClient  # noqa: E402

            self._voices = VoicesClient(client_wrapper=self._client_wrapper)
        return self._voices

    @property
    def conversations(self):
        if self._conversations is None:
            from .conversations.client import ConversationsClient  # noqa: E402

            self._conversations = ConversationsClient(client_wrapper=self._client_wrapper)
        return self._conversations

    @property
    def projects(self):
        if self._projects is None:
            from .projects.client import ProjectsClient  # noqa: E402

            self._projects = ProjectsClient(client_wrapper=self._client_wrapper)
        return self._projects


class AsyncPhonic:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : PhonicEnvironment
        The environment to use for requests from the client. from .environment import PhonicEnvironment



        Defaults to PhonicEnvironment.DEFAULT



    api_key : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phonic import AsyncPhonic

    client = AsyncPhonic(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: PhonicEnvironment = PhonicEnvironment.DEFAULT,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("PHONIC_API_KEY"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            environment=environment,
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._agents: typing.Optional[AsyncAgentsClient] = None
        self._tools: typing.Optional[AsyncToolsClient] = None
        self._extraction_schemas: typing.Optional[AsyncExtractionSchemasClient] = None
        self._voices: typing.Optional[AsyncVoicesClient] = None
        self._conversations: typing.Optional[AsyncConversationsClient] = None
        self._projects: typing.Optional[AsyncProjectsClient] = None

    @property
    def agents(self):
        if self._agents is None:
            from .agents.client import AsyncAgentsClient  # noqa: E402

            self._agents = AsyncAgentsClient(client_wrapper=self._client_wrapper)
        return self._agents

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import AsyncToolsClient  # noqa: E402

            self._tools = AsyncToolsClient(client_wrapper=self._client_wrapper)
        return self._tools

    @property
    def extraction_schemas(self):
        if self._extraction_schemas is None:
            from .extraction_schemas.client import AsyncExtractionSchemasClient  # noqa: E402

            self._extraction_schemas = AsyncExtractionSchemasClient(client_wrapper=self._client_wrapper)
        return self._extraction_schemas

    @property
    def voices(self):
        if self._voices is None:
            from .voices.client import AsyncVoicesClient  # noqa: E402

            self._voices = AsyncVoicesClient(client_wrapper=self._client_wrapper)
        return self._voices

    @property
    def conversations(self):
        if self._conversations is None:
            from .conversations.client import AsyncConversationsClient  # noqa: E402

            self._conversations = AsyncConversationsClient(client_wrapper=self._client_wrapper)
        return self._conversations

    @property
    def projects(self):
        if self._projects is None:
            from .projects.client import AsyncProjectsClient  # noqa: E402

            self._projects = AsyncProjectsClient(client_wrapper=self._client_wrapper)
        return self._projects



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class ConversationAnalysisParams(typing_extensions.TypedDict):
    latencies_ms: typing.Sequence[float]
    """
    Latencies between turns in milliseconds.
    """

    interruptions_count: int
    """
    Number of interruptions in the conversation.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

LanguageCode = typing.Union[
    typing.Literal[
        "ar", "zh", "cs", "da", "nl", "en", "fr", "de", "id", "it", "ja", "ko", "pl", "pt", "ru", "es", "sv", "tr", "vi"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

ConversationEndedBy = typing.Union[
    typing.Literal["user", "user_canceled", "user_validation_failed", "assistant", "error"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ProjectDefaultAgentParams(typing_extensions.TypedDict):
    """
    The project's default agent.
    """

    id: str
    """
    The ID of the project's default agent.
    """

    name: str
    """
    The name of the project's default agent.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.conversation_eval_prompt import ConversationEvalPromptParams


class ProjectsListEvalPromptsResponseParams(typing_extensions.TypedDict):
    conversation_eval_prompts: typing.Sequence[ConversationEvalPromptParams]
